// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.descriptive;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeNode;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.IDescribable;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.view.checklist.ChecklistEditor;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateView;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainerTree;
import eu.etaxonomy.taxeditor.model.IPartContentHasDetails;
import eu.etaxonomy.taxeditor.model.IPartContentHasFactualData;
import eu.etaxonomy.taxeditor.model.IPartContentHasSupplementalData;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.view.AbstractCdmEditorViewPart;
import eu.etaxonomy.taxeditor.view.detail.DetailsViewPart;
import eu.etaxonomy.taxeditor.view.supplementaldata.SupplementalDataViewPart;

/**
 * <p>DescriptiveViewPart class.</p>
 *
 * @author n.hoffmann
 * @created Jun 9, 2010
 * @version 1.0
 */
public class DescriptiveViewPart extends AbstractCdmEditorViewPart implements IPartContentHasDetails, IPartContentHasSupplementalData {

	/** Constant <code>ID="eu.etaxonomy.taxeditor.editor.view.desc"{trunked}</code> */
	public static final String ID = "eu.etaxonomy.taxeditor.editor.view.descriptive";

	protected TreeViewer viewer;

	/**
	 * Maps {@link FeatureNodeContainerTree} to their corresponding {@link TaxonDescritpion}.<br>
	 * This serves as input for the {@link ITreeContentProvider} of the {@link TreeViewer}
	 */
	protected Map<DescriptionBase<?>, FeatureNodeContainerTree> featureNodeContainerCache = new HashMap<DescriptionBase<?>, FeatureNodeContainerTree>();

	protected ToggleDescriptionAction showAllElementsAction;

	protected ToggleDescriptionAction hideAllElementsAction;

	protected int dndOperations = DND.DROP_COPY | DND.DROP_MOVE;

    private DescriptiveContentProvider provider;

	/** {@inheritDoc} */
	@Override
	public void createViewer(Composite parent) {
		viewer = new TreeViewer(new Tree(parent, SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL | SWT.FULL_SELECTION));
		provider = new DescriptiveContentProvider(featureNodeContainerCache);
        viewer.setContentProvider(provider);
		viewer.setLabelProvider(new DescriptiveLabelProvider());
		viewer.setSorter(new DescriptiveViewerSorter());
		viewer.setAutoExpandLevel(2);
		Transfer[] transfers = new Transfer[] { DescriptionElementTransfer.getInstance() };
		viewer.addDragSupport(dndOperations, transfers, new DescriptionElementDragListener(
				this));
		viewer.addDropSupport(dndOperations, transfers,
				new DescriptionElementDropAdapter(viewer));

		// Propagate selection from viewer
		getSite().setSelectionProvider(viewer);

		showAllElementsAction = new ToggleDescriptionAction(false);
		hideAllElementsAction = new ToggleDescriptionAction(true);

		// Add context menu to tree
		createMenu();

		createToolbar();

		// set initial input
//		if(getEditor() != null){
//			viewer.setInput(getEditor().getEditorInput());
//		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.model.AbstractCdmViewPart#getInitialSelection()
	 */
	/** {@inheritDoc} */
	@Override
	protected ISelection getInitialSelection() {
		if(getEditor() != null){
			return new StructuredSelection(getEditor().getEditorInput());
		}

		return super.getInitialSelection();
	}

	protected void createToolbar() {
		IToolBarManager toolBarManager = getViewSite().getActionBars().getToolBarManager();
		toolBarManager.add(showAllElementsAction);
		toolBarManager.add(hideAllElementsAction);
	}

	/*
	 * TODO add to the views menu
	 */
	protected void createMenu(){
		MenuManager menuManager = new MenuManager();
		menuManager.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));

		getSite().registerContextMenu(menuManager, viewer);

		// FIXME for some reason this is not working
		menuManager.add(showAllElementsAction);
		menuManager.add(hideAllElementsAction);

		Control control = viewer.getControl();
		Menu menu = menuManager.createContextMenu(control);

		control.setMenu(menu);
	}

	/** {@inheritDoc} */
	@Override
    protected void selectionChanged_internal(IWorkbenchPart part, ISelection selection) {
		if(AbstractUtility.getActiveEditor() == null){
			showEmptyPage();
			return;
		}

		if(part == this){
		    return;
		}
		if(part instanceof DetailsViewPart || part instanceof SupplementalDataViewPart){
		    // do not show empty page as these views are also used to edit the description selected in this view
		    return;
		}
		// unpackage TreeNode of DerivateView
		else if(part instanceof DerivateView){
		    TreeNode treeNodeOfSelection = EditorUtil.getTreeNodeOfSelection(selection);
		    if(treeNodeOfSelection!=null){
		        selection = new StructuredSelection(treeNodeOfSelection.getValue());
		    }
		}
        else if(part instanceof ChecklistEditor){
            if(selection instanceof StructuredSelection  && ((IStructuredSelection) selection).getFirstElement() instanceof Taxon){
                Taxon taxon = (Taxon)((IStructuredSelection) selection).getFirstElement();
                selection = new StructuredSelection(taxon);
            }
        }
		if(selection instanceof IStructuredSelection
		        && ((IStructuredSelection) selection).getFirstElement() instanceof IDescribable<?>
		        && part instanceof IPartContentHasFactualData){
		    featureNodeContainerCache.clear();
		    showViewer(part, (IStructuredSelection) selection);
		    return;
		}
		else{
		    showEmptyPage();
		}
	}

	/**
	 *
	 * @author n.hoffmann
	 * @created May 28, 2010
	 * @version 1.0
	 */
	protected class ToggleDescriptionAction extends Action{
		private final boolean expanded;

		public ToggleDescriptionAction(boolean expand){
			super(null, IAction.AS_PUSH_BUTTON);
			expanded = expand;
			setImageAndTooltip();
		}

		private void setImageAndTooltip(){
			setImageDescriptor(new ImageDescriptor() {
				@Override
				public ImageData getImageData() {
					setText(expanded ? "Collapse All" : "Expand All");
					String resource = expanded ? ImageResources.COLLAPSE_ALL : ImageResources.EXPAND_ALL;
					return ImageResources.getImage(resource).getImageData();
				}
			});

			String toolTipText = expanded ? "Collapse all" : "Show all factual data";
			setToolTipText(toolTipText);
		}

		@Override
		public void run() {
			if(expanded){
				viewer.collapseAll();
			}else{
				viewer.expandAll();
			}
			setImageAndTooltip();
		}
	}

	/** {@inheritDoc} */
	@Override
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		return super.postOperation(objectAffectedByOperation);
	}

	/** {@inheritDoc} */
	@Override
	public void changed(Object object) {
		for(FeatureNodeContainerTree containerTree : featureNodeContainerCache.values()){
			containerTree.buildTree();
		}
		viewer.refresh();

		if(object instanceof DescriptionElementBase){
			DescriptionElementBase descriptionElement = (DescriptionElementBase) object;
			DescriptionBase description = descriptionElement.getInDescription();
			FeatureNodeContainerTree containerTree = featureNodeContainerCache.get(description);

			FeatureNodeContainer featureNodeContainer = containerTree.getFeatureNodeContainerForDescriptionElement(descriptionElement);
			viewer.expandToLevel(featureNodeContainer, 1);

		}

		if(object != null){
			StructuredSelection selection = new StructuredSelection(object);
			viewer.setSelection(selection, true);
		}

		super.changed(object);
	}

	/** {@inheritDoc} */
	@Override
	public Viewer getViewer() {
		return viewer;
	}

	/**
	 * <p>onComplete</p>
	 *
	 * @return a boolean.
	 */
	@Override
    public boolean onComplete() {
		return false;
	}

	public void toggleShowOnlyIndividualAssociations(){
	    provider.toggleShowOnlyIndividualAssociations();
	    viewer.refresh();
	}
}
